#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
PEACEWORD, Prototype for Extracting And Considering
           the Explainability of WORD embeddings.
           (c) 2025 University of Lille, CNRS

copyright: Peaceword developers (see Authors file),
           GPL v3 License (see LICENSE file)
"""

from methods.solution import Solution
from methods.common import cosine_similarity


def coverage_similarity(vector_a, vector_b, min_d) -> int:
    """
    compute the number of dimensions that are equivalent (according to given min_d)

    Arguments:
    vector_a - the first word vector
    vector_b - the second word vector
    min_d - the minimum distance to consider dimension values as equivalent
    """
    counter = 0
    for value1, value2 in zip(vector_a, vector_b):
        if abs(value1-value2)<=min_d:
                    counter = counter + 1
    return counter

def sub_vector(v, coverage):
    """
    returns the sub vector of v according to a coverage

    Arguments:
    v - the given vector
    coverage - a vector that contains all vector indexes to take into account
    """
    return [v[i] for i in coverage]

def cosine_closest_match(vocab, vector_words, target_word_vector, coverage, banned_words):
    """
    returns the word from the model base that is the closest_match according to cosine_similarity and given coverage

    Arguments:
    vocab - array containing the words including target_word
    vector_words - dictionary of vector words
    target_word_vector - the given target word vector
    coverage - a vector that contains all vector indexes to take into account
    banned_words - list of words to not taking into account
    """ 
    best = None
    best_cosine = None
    for name in vocab:
        if name not in banned_words:
            cosine = cosine_similarity(sub_vector(vector_words[name],coverage), sub_vector(target_word_vector,coverage))
            if best is None:
                best = name
                best_cosine = cosine
            else:
                if cosine > best_cosine:
                    best = name
                    best_cosine = cosine
    return best

def coverage_closest_match(vocab, vector_words, target_word_vector, coverage, banned_words, min_d):
    """
    returns the word from the model base that is the closest_match according to coverage_similarity and given coverage

    Arguments:
    vocab - array containing the words including target_word
    vector_words - dictionary of vector words
    target_word_vector - the given target word vector
    coverage - a vector that contains all vector indexes to take into account
    banned_words - list of words to not taking into account
    min_d - the minimum distance to consider dimension values as equivalent
    """ 
    best=None
    best_match = None
    for name in vocab:
        if name not in banned_words:
            nb = coverage_similarity(sub_vector(vector_words[name],coverage), sub_vector(target_word_vector,coverage),min_d)
            if best is None:
                best = name
                best_match = nb
            else:
                if nb > best_match:
                    best = name
                    best_match = nb
    return best

def test_filter(vector_a, vector_b, vector_size, min_d, min_p):
    """
    this function returns true if it exists at least x percent of dimensions equivalent between vector_a and vector_b,
    returns false otherwise
    Args:
        vector_a: first given word vector
        vector_b: second given word vector
        vector_size : the size of the two given word vectors
        min_d: the value used for comparing dimensions
        min_p : the required percentage of equivalent dimensions to produce a true result

    Returns: true if vectors are considered as equivalent
    """
    if vector_size == 0:
        return False
    counter = coverage_similarity(vector_a, vector_b, min_d)  # coincidence test of word
    return ((counter * 100) / vector_size) >= min_p

def greedy_prepare_data(norm_model, pos_only, min_d, min_p, threshold, target_word):
    """
    Step 1 of the greedy algorithm, returns the vector of words and the pertinent coverage
    :param norm_model: the dataset
    :param pos_only: only manage positive words if equals to True
    :param min_d - the minimum distance  to compare for each dimension of the word vector
    :param min_p - percentage minimum of dimensions (step 1),
                                    if equals to zero, consider all words
    :param threshold - this parameter allows to reduce the coverage for all abs(values) of the target vector > threshold
    if threshold is equals to 0, the resulting coverage contains all dimensions
    :param target_word: the name of the target
    :return: the map of word vectors, the pertinent coverage .
    """

    print("before step 1")
    print("initial number of words:", len(norm_model.keys()))
    print("pos_only:",pos_only)
    print("min_d:",min_d, "min_p:",min_p)
    print("threshold:",threshold)
    # step 1 : complete the base with negative words if pos_only is equal to False
    print("Step 1: ")
    if not pos_only:
        print("complete base with negative words")
    vector_words = {}
    if target_word[0] == "-":
        target_word_vector = -1 * norm_model[target_word[1:]]
    else:
        target_word_vector = norm_model[target_word]
    wv_size = len(target_word_vector)
    if threshold == 0:
        coverage = list(range(wv_size)) # init coverage
    else:
        coverage = []
        for i in range(wv_size):
            if abs(target_word_vector[i]) > threshold:
                coverage.append(i)
    for word in norm_model.keys():
        current_wv = norm_model[word]
        if (word == target_word or (min_p == 0) or
                (test_filter(sub_vector(current_wv, coverage), sub_vector(target_word_vector, coverage),
                                       len(coverage), min_d, min_p))):
            vector_words[word] = current_wv
        if not pos_only:
            inverse_word = f"-{word}"
            inverse_wv = -1 * current_wv # works with np.ndarray
            # inverse_wv = [x * -1 for x in current_wv]
            if (inverse_word == target_word or (min_p == 0) or
                    (test_filter(sub_vector(inverse_wv, coverage), sub_vector(target_word_vector, coverage),
                                          len(coverage), min_d, min_p))):
                vector_words[inverse_word] = inverse_wv
    print("after step 1")
    print("number of words:", len(vector_words))
    print("size of coverage:",len(coverage))
    return vector_words, coverage

def algo_greedy(vector_words, target_word, min_d, max_number_words, test_improve, coverage):
    """
    Greedy algorithm for finding semantically related words

    Arguments:
    model - the model base  provided by the gensim library
    target_word - the given target word
    min_d - the initial minimum distance  to compare for each dimension of the word vector
    max_number_words - max size of the result vector
    test_improve - if True, the  found word (step 2) must improve the solution to be taken into account
    coverage - a vector of index of dimensions to take into account

    returns the found words, the resulting solution and the used coverage size at the end of the process
    """
    solution = Solution([], [], _negative_words_in_dataset=True)
    target_word_vector = vector_words[target_word]
    wv_size = len(target_word_vector)
    vocab = list(vector_words.keys())
    
    iteration = 1
    delta_min_d = (min_d / max_number_words) /2
    best_cs = -1 # init
    banned_words = [ target_word ]

    while iteration <= max_number_words and len(coverage)>0:
        # Step 2 : find the closest word 
        print(f"Step 2 (find the closest word by cosine similarity), iteration number :", iteration)
        print("coverage size:",len(coverage))
        new_word = cosine_closest_match(vocab, vector_words, target_word_vector, coverage,banned_words)
        if new_word is not None:
            improve = True
            banned_words.append(new_word)
            if test_improve: # test if new_word improves the solution
                temp_sol = solution.word_vector(vector_words,wv_size) + vector_words[new_word]
                current_cs = cosine_similarity(temp_sol, target_word_vector)
                if current_cs < best_cs:
                    improve = False
                else:
                    best_cs = current_cs
            if improve:
                solution.add(new_word)
                print("After Step 2 : found word:",new_word)
                # Step 3 :
                print("Step 3 - update coverage")
                coverage = update_coverage(vector_words[new_word], target_word_vector, min_d, coverage)
                print("After Step 3 : coverage size : ", len(coverage))
            else:
                print("After Step 2 : the found word:", new_word, "does not improve the solution")

        iteration = iteration + 1
        min_d = min_d - delta_min_d
    
    return solution, len(coverage)


def update_coverage(word_vector, target_word_vector, min_d, coverage):
    """
      compute a new coverage by subtracting equivalent dimension (according to min_d)
      between a word vector and the target word vector

      Arguments:
      word_vector - the word_vector to compare with the target one
      target_word_vector the target word vector
      min_d  - the minimum delta for each dimension
      coverage - the current coverage
    """
    
    for i,(v1,v2) in enumerate(zip(word_vector,target_word_vector)):
        if i in coverage and abs(v1-v2)<=min_d:
            coverage.remove(i)
    return coverage


