# -*- coding: utf-8 -*-

"""
PEACEWORD, Prototype for Extracting And Considering
           the Explainability of WORD embeddings.
           (c) 2025 University of Lille, CNRS

copyright: Peaceword developers (see Authors file),
           GPL v3 License (see LICENSE file)
"""

import copy

class HillClimbing:
    """
    class that implements the hillclimbing local research algorithm
    """


    def __init__(self, word_embedding, solution, target_word, banned_words=None) -> None:
        """
        class constructor, initialization of instance variables
        :param word_embedding: the class that contains the dataset
        :param solution: an initial solution
        :param target_word: the target word
        :param banned_words: list of words to be banned
        """
        if banned_words is None:
            banned_words = []
        self.model = word_embedding
        self.initial_sol = solution
        self.current_sol = solution
        self.target_word = target_word
        self.vocab = copy.deepcopy(self.model.get_vocab())
        for word in banned_words:
            print(word)
            self.vocab.remove(word)

        self.iterations = 0
        self.evals = 0
        self.eval_method = self.model.evaluate

    def step(self):
        """
        perform one step of the algorithm
        :return: the score of a better solution, returns False if no found better  solution
        """
        score = self.evaluate(self.current_sol, self.target_word)
        for neighbor in self.model.neighbor_solutions(self.current_sol, self.vocab, self.target_word):
            next_score = self.evaluate(neighbor, self.target_word)
            if next_score - score > 0.04:
                score = next_score
                self.set_current_solution(neighbor)
                return score
        return False

    def evaluate(self, sol, predict_word):
        """
        computes an evaluation of a given solution
        :param sol: the current solution
        :param predict_word:  the target word
        :return:
        """
        self.evals += 1
        return self.eval_method(sol, predict_word)

    def set_current_solution(self, new_sol):
        """
        set the current solution
        :param new_sol: the selected solution
        :return:
        """
        self.current_sol = new_sol
        self.iterations +=1

    def get_current_solution(self):
        """
        provides the current solution
        :return:
        """
        return self.current_sol

    def get_iterations(self):
        """
        provides the number of iterations
        """
        return self.iterations

    def __str__(self) -> str:
      return f"HillClimbing (model={self.model}, init={self.initial_sol})"



