# -*- coding: utf-8 -*-
"""
PEACEWORD, Prototype for Extracting And Considering
           the Explainability of WORD embeddings.
           (c) 2025 University of Lille, CNRS

copyright: Peaceword developers (see Authors file),
           GPL v3 License (see LICENSE file)
"""


import copy
import random

from methods.solution import Solution
from methods.common import cosine_similarity


class WordEmbedding:
    """
    class that contains the dataset, provides getter methods and a method for calculating a new neighborhood
    """
    def __init__(self, vectors, pos_only, seed) -> None:
        """
        class constructor, initialize the instance variables
        :param vectors: KeyedVectors object from gensim library
        :param pos_only: boolean value, restricted to  positive words if set to True
        :param seed: the random seed
        """
        self.wv = vectors
        self.pos_only = pos_only
        self.list_keys = list(self.wv.keys())
        random.seed(seed)

    def get_vocab_size(self):
        """ provides the dataset size """
        return len(self.list_keys)

    def get_vocab(self):
        """ provides the list of words managed by the dataset"""
        return self.list_keys

    def get_random_word(self):
        """ provides a random word from the data set """
        return random.choice(self.list_keys)

    def random_solution(self, nb_words, eval_word):
        """ provides a random solution containing nb_words
        Arguments:
            nb_words : the solution size to generate
            eval_word : the target word, this word must not be in the solution
        """
        positives = []
        negatives = []
        couple = (positives, negatives)
        for i in range(nb_words):
            new_word = self.get_random_word()
            while new_word in (positives+negatives) or new_word == eval_word:
                new_word = self.get_random_word()
            if self.pos_only:
                positives.append(new_word)
            else:
                random.choice(couple).append(new_word)
        return Solution(positives, negatives)

    def evaluate(self, solution, predict_word):
        """
        provides the cosine_similarity of the solution and  the target word
        :param solution: a solution
        :param predict_word: the target word
        :return: the cosine similarity
        """
        result = solution.word_vector(self.wv, len(self.wv[predict_word]))

        return cosine_similarity(result, self.wv[predict_word])

    def neighbor_solutions(self, solution, vocab, eval_word):
        """
        calculates a list of neighboring solutions from a given solution;
        neighboring solutions differ from the initial solutions by a single word
        :param solution: the current solution
        :param vocab: vocabulary
        :param eval_word: the target word
        """
        neighbors = []
        #part 1 : removing a word
        for word in solution.positive:
            s_copy = copy.deepcopy(solution)
            s_copy.positive.remove(word)
            neighbors.append(s_copy)
        if not self.pos_only:
            for word in solution.negative:
                s_copy = copy.deepcopy(solution)
                s_copy.negative.remove(word)
                neighbors.append(s_copy)

        #part 2 : changing a word
        for word in solution.positive:
            s_copy = copy.deepcopy(solution)
            s_copy.positive.remove(word)
            for v in vocab:
                if v not in (solution.positive+solution.negative) and v != eval_word and v != word:
                    replaced = copy.deepcopy(s_copy)
                    replaced.positive.append(v)
                    neighbors.append(replaced)
        if not self.pos_only:
            for word in solution.negative:
                s_copy = copy.deepcopy(solution)
                s_copy.negative.remove(word)
                for v in vocab:
                    if v not in (solution.positive + solution.negative) and v != eval_word and v != word:
                        replaced = copy.deepcopy(s_copy)
                        replaced.negative.append(v)
                        neighbors.append(replaced)

        # part 3 : add a word if size of solution < 6
        if solution.size() < 6:
            for word in vocab:
                if word not in (solution.positive + solution.negative) and word != eval_word:
                    s_copy = copy.deepcopy(solution)
                    s_copy.positive.append(word)
                    neighbors.append(s_copy)
                if not self.pos_only and word not in (solution.positive + solution.negative) and word != eval_word:
                    s_copy = copy.deepcopy(solution)
                    s_copy.negative.append(word)
                    neighbors.append(s_copy)

        # remove empty solutions
        for n in neighbors:
            if n.is_empty():
                neighbors.remove(n)

        return neighbors