#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
PEACEWORD, Prototype for Extracting And Considering
           the Explainability of WORD embeddings.
           (c) 2025 University of Lille, CNRS

copyright: Peaceword developers (see Authors file),
           GPL v3 License (see LICENSE file)
"""
import sys
import csv
import re
import os
import time
from filelock import FileLock

from gensim.models.word2vec import Word2Vec
from gensim.models import KeyedVectors

from methods.greedy.greedy import algo_greedy, greedy_prepare_data
from methods.common import cosine_similarity, normalize_model



def main():
    if len(sys.argv) != 8:
        print('Usage: python3 run_greedy.py dataset_location  pos_only (yes or no) ',
              ' min_d (real), min_p (0-10) '
              'threshold (0 or limit value)  test_improve (yes or no) eval_word')
        sys.exit()

    # parameters :

    max_number_words = 6

    dataset_location = sys.argv[1]
    param_pos_only = sys.argv[2]
    param_min_d = sys.argv[3]
    param_min_p = sys.argv[4]
    param_threshold = sys.argv[5]
    param_test_improve = sys.argv[6]
    target_word=sys.argv[7]

    regex = '[+-]?[0-9]+.?[0-9]*'

    if param_pos_only not in ['yes','no']:
        print(f'bad pos_only parameter : {param_pos_only} (yes or no are only supported)')
        sys.exit()
    if not re.search(regex, param_threshold):
        print(f'bad threshold parameter : {param_threshold} must be a number')
        sys.exit()
    if not re.search(regex, param_min_d):
        print(f'bad min_d parameter : {param_min_d} must be a number')
        sys.exit()
    min_d = float(param_min_d)
    if not re.search(regex, param_min_p):
        print(f'bad min_p parameter : {param_min_p} must be a number')
        sys.exit()
    if param_test_improve not in ['yes','no']:
        print(f'bad test_improve parameter : {param_test_improve} (yes or no are only supported)')
        sys.exit()

    path, filename = os.path.split(dataset_location)
    if "glove" in dataset_location:
        model = KeyedVectors.load_word2vec_format(dataset_location, binary=True)
    else:
        model = Word2Vec.load(dataset_location)
        model = model.wv

    norm_model = normalize_model(model)
    start_time = time.time()
    vector_words, coverage =greedy_prepare_data (norm_model,
                                                (param_pos_only=='yes'),
                                                min_d, float(param_min_p),
                                                float(param_threshold),target_word)

    word_size = len(vector_words[target_word])
    nb_unused_dimensions = word_size - len(coverage)
    print(f"unused_dimensions: {nb_unused_dimensions}")

    solution, last_coverage_size = algo_greedy(vector_words, target_word, min_d,
                                       max_number_words, (param_test_improve=='yes'), coverage)
    end_time = time.time()
    print(f"found solution for target {target_word} :", solution)


    run_time=end_time - start_time
    print(f"Total execution time: {run_time} seconds")
    print("solution:", solution)

    cs = cosine_similarity(solution.word_vector(vector_words,word_size), vector_words[target_word])
    print(f"cosine similarity of solution :{cs}")

    print(f"last coverage size:{last_coverage_size}")

    file_name = f"greedy_{filename}_{target_word}.csv"
    lock = FileLock(file_name + ".lock")  # lock file
    with lock:
        with open(file_name, "a", newline='', encoding='utf-8') as csvfile:
            fwriter = csv.writer(csvfile, delimiter=';', quoting=csv.QUOTE_NONE, escapechar='@')
            fwriter.writerow([filename, target_word, last_coverage_size, str(run_time), cs,
                              param_pos_only, param_min_p, param_min_d,
                              param_threshold, param_test_improve,solution,nb_unused_dimensions])
            csvfile.close()
    print(filename, target_word, last_coverage_size
          , str(run_time), cs,
          param_pos_only, param_min_p,param_min_d,
          param_threshold, param_test_improve, solution, nb_unused_dimensions)


if __name__ == '__main__':
    main()