#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
PEACEWORD, Prototype for Extracting And Considering
           the Explainability of WORD embeddings.
           (c) 2025 University of Lille, CNRS

copyright: Peaceword developers (see Authors file),
           GPL v3 License (see LICENSE file)
"""


from gensim.models.word2vec import Word2Vec
from gensim.models import KeyedVectors

import time
import sys
import csv
import os
from filelock import FileLock

from methods.hillclimbing.utils import WordEmbedding
from methods.hillclimbing.hillclimbing import HillClimbing
from methods.common import normalize_model

def main():
    if len(sys.argv) != 5:
        print('Usage: python3 run_hillclimbing.py dataset_location pos_only seed eval_word')
        sys.exit()
    dataset_location = sys.argv[1]
    path, dataset_name = os.path.split(dataset_location)
    param_pos_only = sys.argv[2]
    seed = sys.argv[3]
    eval_word = sys.argv[4]
    if param_pos_only not in ['yes','no']:
        print(f'bad pos_only parameter : {param_pos_only} (yes or no are only supported)')
        sys.exit()
    else:
        pos_only = (param_pos_only == 'yes')

    if "glove" in dataset_location:
        model = KeyedVectors.load_word2vec_format(dataset_location, binary=True)
    else:
        model = Word2Vec.load(dataset_location)
        model = model.wv

    norm_model = normalize_model(model)
    print("INFO :: model Trained")
    print("run No", seed)
    word_embedding = WordEmbedding(norm_model, pos_only, seed)

    start_time = time.time()
    start_sol = word_embedding.random_solution(6, eval_word)
    hc = HillClimbing(word_embedding, start_sol, eval_word, None)
    step = True
    print("starting hc")
    score = 0
    while step:
        step = hc.step()
        if not step == False:
            score = step
            print(f"{str(hc.get_current_solution())} : score {step}")
    end_time = time.time()
    print("--------------")
    print(f"{str(hc.get_current_solution())} : score {score}")
    print(f"iterations : {hc.iterations}")

    run_time = end_time - start_time
    print(f"Total execution time: {run_time} seconds")
    file_name = f"hillclimbing_{dataset_name}_{param_pos_only}_{eval_word}.csv"
    lock = FileLock(file_name + ".lock")  # lock file
    with lock:
        with open(file_name, "a", newline='', encoding='utf-8') as csvfile:
            fwriter = csv.writer(csvfile, delimiter=';', quoting=csv.QUOTE_NONE, escapechar='@')
            fwriter.writerow([seed, str(start_sol), eval_word, str(score), hc.iterations,
                              str(run_time), str(hc.get_current_solution())])
            csvfile.close()
    print(seed, start_sol, eval_word, score, hc.iterations, run_time, hc.get_current_solution())


if __name__ == '__main__':
    main()

